# -*- coding: utf-8 -*-
"""
Créé en juin 2023
auteur : David ALBERTO (www.astrolabe-science.fr)

Tableau donnant l'ascension droite et la déclinaison des planètes, pour une année.
"""
import matplotlib.pyplot as plt
import numpy as np
import datetime as dt
import pandas as pd
from skyfield.api import load, utc

# --------------------------------- REGLAGES UTILISATEUR :
annee = 2023
plt.rcParams["font.family"] = "Roboto"              # ou autre police installée sur l'ordinateur
couleurFond='#c7cef2'                               # couleur d'arrière-plan de la figure
NomFichier = "EphemeridesPlanetes" + str(annee)
c_AD = 'green' # couleur de l'ascension droite
c_decl = 'darkviolet' # couleur de la déclinaison
x_AD = 0.40 # abscisses ascension droite
x_decl = 0.70 # abscisse déclinaison

# FIN DES REGLAGES UTILISATEURS
ts = load.timescale()#crée une échelle de dates Skyfield

#%%%%%%%%%%%%%%%%%%% Fonctions
def coordonnees(planete, date):
    """
    planete : astre défini par Skyfield
    date : date ou série de dates Skyfield
    renvoie l'ascension droite et la déclinaison (formats Skyfield).
    """
    position = terre.at(date).observe(planete).apparent()
    AD, decl, _ = position.radec()
    return AD, decl

def formatage_AD(AD):
    # renvoie un string de l'ascension droite'
    AD_str = str(AD)
    AD_str = AD_str.split()
    AD_str =AD_str[0] + ' ' +AD_str[1]
    return AD_str

def formatage_decl(decl):
    # renvoie un string de la déclinaison
    decl_str = str(round(decl.degrees,1)).replace('.', ',')
    return decl_str

def nom_planete(planete):
    """
    planete : astre défini par Skyfield
    """
    if planete==mercure:
        nom = 'Mercure'
    elif planete == venus:
        nom = 'Vénus'
    elif planete == mars:
        nom = 'Mars'
    elif planete == jupiter:
        nom = 'Jupiter'
    elif planete == saturne:
        nom = 'Saturne'
    elif planete == uranus:
        nom = 'Uranus'
    elif planete == neptune:
        nom = 'Neptune'
    return nom

def formatage_axe(axe):
    """
    formate l'axe du mois.
    """
    axe.set_xticks([])
    axe.invert_yaxis()
    axe.set_ylim(len(liste_dates)+0.5,-0.5)
    if axe == axes[0]:
        axe.set_yticks(np.arange(24,0,-1),liste_grad)
    else:axe.set_yticks([])

#%%%%%%%%%%
# (Télé)chargement du fichier d'ephémérides de421.bsp (16 MB, période 1849-2050)
eph = load('../skyfield-ephemerides/de421.bsp')# si le fichier est déjà chargé sur le disque (chemin à adapter).
# eph=api.load('de421.bsp')# pour télécharger automatiquement le fichier, dans le dossier du fichier Python

mercure = eph['mercury']
venus = eph['venus']
terre= eph['earth']
mars = eph['mars']
jupiter = eph['jupiter barycenter']
saturne = eph['saturn barycenter']
uranus = eph['uranus barycenter']
neptune = eph['neptune barycenter']

liste_planetes = [mercure, venus, mars, jupiter, saturne, uranus, neptune]
N_planetes = len(liste_planetes) # nombre de planetes

noms_planetes = [nom_planete(planete) for planete in liste_planetes]

#Paramètres du graphique : --------------------------------------

fig, axes = plt.subplots(ncols = N_planetes,figsize = (29.7/2.54, 21/2.54), tight_layout=True, facecolor=couleurFond)

axes = axes.flatten() # rend les axes itérables
plt.suptitle(f"Coordonnées équatoriales des planètes - {annee}", fontsize=16, fontweight='bold')

# création de la liste de dates (1er et 15e jour du mois) :
premier = np.array([dt.datetime(annee, i, 1,tzinfo=utc) for i in range(1,13)])
quinze = np.array([dt.datetime(annee, i, 15,tzinfo=utc) for i in range(1,13)])
liste_dates = np.concatenate((premier,quinze))
liste_dates = np.sort(liste_dates) # classement dans l'ordre

liste_grad = ['','D','','N','','O','','S','','A','','J','','J','','M','','A','','M','','F','','J']

# création d'un DataFrame 'table' pour collecter et exporter les données :
colonnes_table = []
for nom in noms_planetes:
    colonnes_table.append(nom + '_AD')
    colonnes_table.append(nom + '_decl')
table = pd.DataFrame(index=liste_dates, columns=colonnes_table)


for N, planete in enumerate(liste_planetes):
    axe = axes[N-1]
    formatage_axe(axe)
    planete = liste_planetes[N-1]
    nom = nom_planete(planete)
    axe.set_title(nom)
    axe.text(x_AD,0.25,'AD', va='center',ha='center',c=c_AD, fontweight='bold')
    axe.text(x_decl,0.25,'décl.', va='center',ha='center',c=c_decl, fontweight='bold')
    axe.plot([0.5*x_AD,1.2*x_decl],[0.55,0.55],c='darkblue',lw=0.7) # ligne horizontale
    for i, date in enumerate(liste_dates):
        date_sk = ts.from_datetime(date) # date format Skyfield
        AD, decl = coordonnees(planete, date_sk)
        decl_str = formatage_decl(decl)
        AD_str = formatage_AD(AD)
        axe.text(x_AD,i+1,AD_str, va='center',ha='center',c=c_AD, fontweight='bold')
        axe.text(x_decl,i+1,decl_str, va='center',ha='center',c=c_decl, fontweight='bold')
        # collecte des angles dans le DataFrame :
        table.iloc[i][(2*N-1)-1] = AD.hours
        table.iloc[i][(2*N-1)] = decl.degrees

# export du DataFrame dans un fichier csv externe :
table.to_csv('ephemPlanetes'+str(annee)+'.csv', index_label='date')
#----------------------------------------------------------------------------

# signature
fig.text(0.98,0.04,'David ALBERTO (www.astrolabe-science.fr)',rotation=90,
        ha='center',va='bottom',transform=fig.transFigure, fontsize=6)
fig.text(0.08,0.007,'les 1er et 15e du mois',
        ha='center',va='bottom',transform=fig.transFigure, fontsize=8)

fig.savefig(NomFichier + ".png", dpi=200)
fig.savefig(NomFichier + ".pdf")
# plt.show()
